/*
 * Copyright (C) 2015 Stuart Howarth <showarth@marxoft.co.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "dailymotionvideomodel.h"
#include "dailymotion.h"
#include "dailymotionplaylist.h"
#ifdef CUTETUBE_DEBUG
#include <QDebug>
#endif

DailymotionVideoModel::DailymotionVideoModel(QObject *parent) :
    QAbstractListModel(parent),
    m_request(new QDailymotion::ResourcesRequest(this)),
    m_hasMore(false)
{
    m_roles[DateRole] = "date";
    m_roles[DescriptionRole] = "description";
    m_roles[DurationRole] = "duration";
    m_roles[FavouriteRole] = "favourited";
    m_roles[IdRole] = "id";
    m_roles[LargeThumbnailUrlRole] = "largeThumbnailUrl";
    m_roles[ThumbnailUrlRole] = "thumbnailUrl";
    m_roles[TitleRole] = "title";
    m_roles[UrlRole] = "url";
    m_roles[UserIdRole] = "userId";
    m_roles[UsernameRole] = "username";
    m_roles[ViewCountRole] = "viewCount";
#if QT_VERSION < 0x050000
    setRoleNames(m_roles);
#endif
    m_request->setClientId(Dailymotion::instance()->clientId());
    m_request->setClientSecret(Dailymotion::instance()->clientSecret());
    m_request->setAccessToken(Dailymotion::instance()->accessToken());
    m_request->setRefreshToken(Dailymotion::instance()->refreshToken());
    
    connect(m_request, SIGNAL(accessTokenChanged(QString)), Dailymotion::instance(), SLOT(setAccessToken(QString)));
    connect(m_request, SIGNAL(refreshTokenChanged(QString)), Dailymotion::instance(), SLOT(setRefreshToken(QString)));
    connect(m_request, SIGNAL(finished()), this, SLOT(onRequestFinished()));
}

QString DailymotionVideoModel::errorString() const {
    return Dailymotion::getErrorString(m_request->result().toMap());
}

QDailymotion::ResourcesRequest::Status DailymotionVideoModel::status() const {
    return m_request->status();
}

#if QT_VERSION >=0x050000
QHash<int, QByteArray> DailymotionVideoModel::roleNames() const {
    return m_roles;
}
#endif

int DailymotionVideoModel::rowCount(const QModelIndex &) const {
    return m_items.size();
}

bool DailymotionVideoModel::canFetchMore(const QModelIndex &) const {
    return (status() != QDailymotion::ResourcesRequest::Loading) && (m_hasMore);
}

void DailymotionVideoModel::fetchMore(const QModelIndex &) {
    if (!canFetchMore()) {
        return;
    }
    
    const int page = m_filters.value("page").toInt();
    m_filters["page"] = (page > 0 ? page + 1 : 2);
    m_request->list(m_resourcePath, m_filters, Dailymotion::VIDEO_FIELDS);
    emit statusChanged(status());
}

QVariant DailymotionVideoModel::data(const QModelIndex &index, int role) const {
    if (DailymotionVideo *video = get(index.row())) {
        return video->property(m_roles[role]);
    }
    
    return QVariant();
}

QMap<int, QVariant> DailymotionVideoModel::itemData(const QModelIndex &index) const {
    QMap<int, QVariant> map;
    
    if (DailymotionVideo *video = get(index.row())) {
        QHashIterator<int, QByteArray> iterator(m_roles);
        
        while (iterator.hasNext()) {
            iterator.next();
            map[iterator.key()] = video->property(iterator.value());
        }
    }
    
    return map;
}

QVariant DailymotionVideoModel::data(int row, const QByteArray &role) const {
    if (DailymotionVideo *video = get(row)) {
        return video->property(role);
    }
    
    return QVariant();
}

QVariantMap DailymotionVideoModel::itemData(int row) const {
    QVariantMap map;
    
    if (DailymotionVideo *video = get(row)) {
        foreach (QByteArray role, m_roles.values()) {
            map[role] = video->property(role);
        }
    }
    
    return map;
}

DailymotionVideo* DailymotionVideoModel::get(int row) const {
    if ((row >= 0) && (row < m_items.size())) {
        return m_items.at(row);
    }
    
    return 0;
}

void DailymotionVideoModel::list(const QString &resourcePath, const QVariantMap &filters) {
    if (status() == QDailymotion::ResourcesRequest::Loading) {
        return;
    }
    
    clear();
    m_resourcePath = resourcePath;
    m_filters = filters;
    m_request->list(resourcePath, filters, Dailymotion::VIDEO_FIELDS);
    emit statusChanged(status());
    
    disconnect(Dailymotion::instance(), 0, this, 0);
        
    if (resourcePath == "/me/favorites") {
        connect(Dailymotion::instance(), SIGNAL(videoFavourited(DailymotionVideo*)),
                this, SLOT(onVideoFavourited(DailymotionVideo*)));
        connect(Dailymotion::instance(), SIGNAL(videoUnfavourited(DailymotionVideo*)),
                this, SLOT(onVideoUnfavourited(DailymotionVideo*)));
    }
    else if (resourcePath.section('/', -3, -3) == "playlist") {
        connect(Dailymotion::instance(), SIGNAL(videoAddedToPlaylist(DailymotionVideo*, DailymotionPlaylist*)),
                this, SLOT(onVideoAddedToPlaylist(DailymotionVideo*, DailymotionPlaylist*)));
        connect(Dailymotion::instance(), SIGNAL(videoRemovedFromPlaylist(DailymotionVideo*, DailymotionPlaylist*)),
                this, SLOT(onVideoRemovedFromPlaylist(DailymotionVideo*, DailymotionPlaylist*)));
    }
}

void DailymotionVideoModel::clear() {
    if (!m_items.isEmpty()) {
        beginResetModel();
        qDeleteAll(m_items);
        m_items.clear();
        m_hasMore = false;
        endResetModel();
        emit countChanged(rowCount());
    }
}

void DailymotionVideoModel::cancel() {
    m_request->cancel();
}

void DailymotionVideoModel::reload() {
    clear();
    m_request->list(m_resourcePath, m_filters, Dailymotion::VIDEO_FIELDS);
    emit statusChanged(status());
}

void DailymotionVideoModel::append(DailymotionVideo *video) {
    beginInsertRows(QModelIndex(), m_items.size(), m_items.size());
    m_items << video;
    endInsertRows();
}

void DailymotionVideoModel::insert(int row, DailymotionVideo *video) {
    if ((row >= 0) && (row < m_items.size())) {
        beginInsertRows(QModelIndex(), row, row);
        m_items.insert(row, video);
        endInsertRows();
    }
    else {
        append(video);
    }
}

void DailymotionVideoModel::remove(int row) {
    if ((row >= 0) && (row < m_items.size())) {
        beginRemoveRows(QModelIndex(), row, row);
        m_items.takeAt(row)->deleteLater();
        endRemoveRows();
    }
}

void DailymotionVideoModel::onRequestFinished() {
    if (m_request->status() == QDailymotion::ResourcesRequest::Ready) {
        QVariantMap result = m_request->result().toMap();
        
        if (!result.isEmpty()) {
            m_hasMore = result.value("has_more").toBool();
            QVariantList list = result.value("list").toList();

            beginInsertRows(QModelIndex(), m_items.size(), m_items.size() + list.size() - 1);
    
            foreach (QVariant item, list) {
                m_items << new DailymotionVideo(item.toMap(), this);
            }

            endInsertRows();
            emit countChanged(rowCount());
        }
    }
    
    emit statusChanged(status());
}

void DailymotionVideoModel::onVideoAddedToPlaylist(DailymotionVideo *video, DailymotionPlaylist *playlist) {
    if (m_resourcePath.section('/', -2, -2) == playlist->id()) {
        insert(0, new DailymotionVideo(video, this));
    }
#ifdef CUTETUBE_DEBUG
    qDebug() << "DailymotionVideoModel::onVideoAddedToPlaylist" << video->id() << playlist->id();
#endif
}

void DailymotionVideoModel::onVideoRemovedFromPlaylist(DailymotionVideo *video, DailymotionPlaylist *playlist) {
    if (m_resourcePath.section('/', -2, -2) == playlist->id()) {
        QModelIndexList list = match(index(0), IdRole, video->id(), 1, Qt::MatchExactly);
        
        if (!list.isEmpty()) {
            remove(list.first().row());
        }
    }
#ifdef CUTETUBE_DEBUG
    qDebug() << "DailymotionVideoModel::onVideoRemovedFromPlaylist" << video->id() << playlist->id();
#endif
}

void DailymotionVideoModel::onVideoFavourited(DailymotionVideo *video) {
    insert(0, new DailymotionVideo(video, this));
#ifdef CUTETUBE_DEBUG
    qDebug() << "DailymotionVideoModel::onVideoFavourited" << video->id();
#endif
}

void DailymotionVideoModel::onVideoUnfavourited(DailymotionVideo *video) {
    QModelIndexList list = match(index(0), IdRole, video->id(), 1, Qt::MatchExactly);
    
    if (!list.isEmpty()) {
        remove(list.first().row());
    }
#ifdef CUTETUBE_DEBUG
    qDebug() << "DailymotionVideoModel::onVideoUnfavourited" << video->id();
#endif
}
